/*
 * SIM800HostCommunication.cpp
 * 
 * Copyright 2014 Stefan Mavrodiev <support@olimex.com>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 */
#include "Arduino.h"
#include <SIM800HostCommunication.hpp>


SIM800HostCommunication::SIM800HostCommunication()
{
}

void SIM800HostCommunication::begin(uint16_t baudRate)
{
    Serial1.begin(baudRate);
    while(!Serial1);
    
    //~ delay(10);
    //~ flush();
    //~ delay(10);
    //~ echo(false);
    
}

String SIM800HostCommunication::writeCommand(String ATcommand, uint16_t responceDelay)
{
    String responce = "";
 
    //Clear Serial1
    flush();
    
    //Send the command
    Serial1.print(ATcommand);
    
    if(responceDelay == 0)
        return responce;
        
    //Wait for responce
    delay(responceDelay);
    
    //Read the responce
    while(Serial1.available() > 0){
        char c = Serial1.read();
        responce += c;
    }
    return responce;
}

void SIM800HostCommunication::flush()
{
        while(Serial1.available() > 0)
            Serial1.read();
}

bool SIM800HostCommunication::checkResponce(String responce)
{
    responce.trim();
    
    if(responce.equals("OK")){
        return true;
    }else if(responce.equals("ERROR")){
        return false;
    }else{
#ifdef __DEBUG__
        Serial.print("Unknown responce: ");
        Serial.println(responce);
#endif
        return false;
    }
}

String SIM800HostCommunication::readEvent(uint16_t eventDelay)
{
    
    String event = "";
    bool CRdetected = false;
    
    delay(eventDelay);
    
    do{
        while(Serial1.available() > 0){
            char c = Serial1.read();
            if(c == 13)
                continue;
            if(c == 10){
                if(event == ""){
                    continue;
                }else{
                    CRdetected = true;
                    break;
                }
            }
            event += c;
        }
    }while(!CRdetected);
    
        
    return event;
}

void SIM800HostCommunication::echo(bool enabled)
{
    if(enabled)
        writeCommand("ATE1\r", 100);
    else
        writeCommand("ATE0\r", 100);
        
    flush();
}

void SIM800HostCommunication::write(String text)
{
    Serial1.print(text);
}
